@@ -92,7 +92,7 @@ func Execute(args []string) Response {
 
 	if c == cmd && hugoCmd.c != nil {
 		// Root command executed
-		resp.Result = hugoCmd.c.hugo
+		resp.Result = hugoCmd.c.hugo()
 	}
 
 	if err == nil {
@@ -134,10 +134,14 @@ func (c *commandeer) createLogger(cfg config.Provider, running bool) (*loggers.L
 		logHandle       = ioutil.Discard
 		logThreshold    = jww.LevelWarn
 		logFile         = cfg.GetString("logFile")
-		outHandle       = os.Stdout
+		outHandle       = ioutil.Discard
 		stdoutThreshold = jww.LevelWarn
 	)
 
+	if !c.h.quiet {
+		outHandle = os.Stdout
+	}
+
 	if c.h.verboseLog || c.h.logging || (c.h.logFile != "") {
 		var err error
 		if logFile != "" {
@@ -338,16 +342,16 @@ func (c *commandeer) fullBuild() error {
 		}
 	}
 
-	for _, s := range c.hugo.Sites {
+	for _, s := range c.hugo().Sites {
 		s.ProcessingStats.Static = langCount[s.Language().Lang]
 	}
 
 	if c.h.gc {
-		count, err := c.hugo.GC()
+		count, err := c.hugo().GC()
 		if err != nil {
 			return err
 		}
-		for _, s := range c.hugo.Sites {
+		for _, s := range c.hugo().Sites {
 			// We have no way of knowing what site the garbage belonged to.
 			s.ProcessingStats.Cleaned = uint64(count)
 		}
@@ -463,8 +467,6 @@ func (c *commandeer) initProfiling() (func(), error) {
 }
 
 func (c *commandeer) build() error {
-	defer c.timeTrack(time.Now(), "Total")
-
 	stopProfiling, err := c.initProfiling()
 	if err != nil {
 		return err
@@ -483,7 +485,7 @@ func (c *commandeer) build() error {
 	// TODO(bep) Feedback?
 	if !c.h.quiet {
 		fmt.Println()
-		c.hugo.PrintProcessingStats(os.Stdout)
+		c.hugo().PrintProcessingStats(os.Stdout)
 		fmt.Println()
 
 		if createCounter, ok := c.destinationFs.(hugofs.DuplicatesReporter); ok {
@@ -519,7 +521,7 @@ func (c *commandeer) build() error {
 }
 
 func (c *commandeer) serverBuild() error {
-	defer c.timeTrack(time.Now(), "Total")
+	defer c.timeTrack(time.Now(), "Built")
 
 	stopProfiling, err := c.initProfiling()
 	if err != nil {
@@ -539,7 +541,7 @@ func (c *commandeer) serverBuild() error {
 	// TODO(bep) Feedback?
 	if !c.h.quiet {
 		fmt.Println()
-		c.hugo.PrintProcessingStats(os.Stdout)
+		c.hugo().PrintProcessingStats(os.Stdout)
 		fmt.Println()
 	}
 
@@ -558,7 +560,7 @@ func (c *commandeer) doWithPublishDirs(f func(sourceFs *filesystems.SourceFilesy
 
 	langCount := make(map[string]uint64)
 
-	staticFilesystems := c.hugo.BaseFs.SourceFilesystems.Static
+	staticFilesystems := c.hugo().BaseFs.SourceFilesystems.Static
 
 	if len(staticFilesystems) == 0 {
 		c.logger.INFO.Println("No static directories found to sync")
@@ -610,7 +612,7 @@ func chmodFilter(dst, src os.FileInfo) bool {
 }
 
 func (c *commandeer) copyStaticTo(sourceFs *filesystems.SourceFilesystem) (uint64, error) {
-	publishDir := c.hugo.PathSpec.PublishDir
+	publishDir := c.hugo().PathSpec.PublishDir
 	// If root, remove the second '/'
 	if publishDir == "//" {
 		publishDir = helpers.FilePathSeparator
@@ -655,13 +657,10 @@ func (c *commandeer) copyStaticTo(sourceFs *filesystems.SourceFilesystem) (uint6
 }
 
 func (c *commandeer) firstPathSpec() *helpers.PathSpec {
-	return c.hugo.Sites[0].PathSpec
+	return c.hugo().Sites[0].PathSpec
 }
 
 func (c *commandeer) timeTrack(start time.Time, name string) {
-	if c.h.quiet {
-		return
-	}
 	elapsed := time.Since(start)
 	c.logger.FEEDBACK.Printf("%s in %v ms", name, int(1000*elapsed.Seconds()))
 }
@@ -689,7 +688,7 @@ func (c *commandeer) getDirList() ([]string, error) {
 
 	}
 
-	watchDirs := c.hugo.PathSpec.BaseFs.WatchDirs()
+	watchDirs := c.hugo().PathSpec.BaseFs.WatchDirs()
 	for _, watchDir := range watchDirs {
 
 		w := hugofs.NewWalkway(hugofs.WalkwayConfig{Logger: c.logger, Info: watchDir, WalkFn: walkFn})
@@ -704,7 +703,7 @@ func (c *commandeer) getDirList() ([]string, error) {
 }
 
 func (c *commandeer) buildSites() (err error) {
-	return c.hugo.Build(hugolib.BuildCfg{})
+	return c.hugo().Build(hugolib.BuildCfg{})
 }
 
 func (c *commandeer) handleBuildErr(err error, msg string) {
@@ -719,6 +718,9 @@ func (c *commandeer) handleBuildErr(err error, msg string) {
 
 func (c *commandeer) rebuildSites(events []fsnotify.Event) error {
 	defer c.timeTrack(time.Now(), "Total")
+	defer func() {
+		c.wasError = false
+	}()
 
 	c.buildErr = nil
 	visited := c.visitedURLs.PeekAllSet()
@@ -726,25 +728,28 @@ func (c *commandeer) rebuildSites(events []fsnotify.Event) error {
 
 		// Make sure we always render the home pages
 		for _, l := range c.languages {
-			langPath := c.hugo.PathSpec.GetLangSubDir(l.Lang)
+			langPath := c.hugo().PathSpec.GetLangSubDir(l.Lang)
 			if langPath != "" {
 				langPath = langPath + "/"
 			}
-			home := c.hugo.PathSpec.PrependBasePath("/"+langPath, false)
+			home := c.hugo().PathSpec.PrependBasePath("/"+langPath, false)
 			visited[home] = true
 		}
 
 	}
-	return c.hugo.Build(hugolib.BuildCfg{RecentlyVisited: visited}, events...)
+	return c.hugo().Build(hugolib.BuildCfg{RecentlyVisited: visited, ErrRecovery: c.wasError}, events...)
 }
 
 func (c *commandeer) partialReRender(urls ...string) error {
+	defer func() {
+		c.wasError = false
+	}()
 	c.buildErr = nil
 	visited := make(map[string]bool)
 	for _, url := range urls {
 		visited[url] = true
 	}
-	return c.hugo.Build(hugolib.BuildCfg{RecentlyVisited: visited, PartialReRender: true})
+	return c.hugo().Build(hugolib.BuildCfg{RecentlyVisited: visited, PartialReRender: true, ErrRecovery: c.wasError})
 }
 
 func (c *commandeer) fullRebuild(changeType string) {
@@ -773,9 +778,9 @@ func (c *commandeer) fullRebuild(changeType string) {
 			time.Sleep(2 * time.Second)
 		}()
 
-		defer c.timeTrack(time.Now(), "Total")
+		defer c.timeTrack(time.Now(), "Rebuilt")
 
-		c.commandeerHugoState = &commandeerHugoState{}
+		c.commandeerHugoState = newCommandeerHugoState()
 		err := c.loadConfig(true, true)
 		if err != nil {
 			// Set the processing on pause until the state is recovered.
@@ -951,7 +956,7 @@ func (c *commandeer) handleEvents(watcher *watcher.Batcher,
 	filtered := []fsnotify.Event{}
 	for _, ev := range evs {
 		// Check the most specific first, i.e. files.
-		contentMapped := c.hugo.ContentChanges.GetSymbolicLinkMappings(ev.Name)
+		contentMapped := c.hugo().ContentChanges.GetSymbolicLinkMappings(ev.Name)
 		if len(contentMapped) > 0 {
 			for _, mapped := range contentMapped {
 				filtered = append(filtered, fsnotify.Event{Name: mapped, Op: ev.Op})
@@ -963,7 +968,7 @@ func (c *commandeer) handleEvents(watcher *watcher.Batcher,
 
 		dir, name := filepath.Split(ev.Name)
 
-		contentMapped = c.hugo.ContentChanges.GetSymbolicLinkMappings(dir)
+		contentMapped = c.hugo().ContentChanges.GetSymbolicLinkMappings(dir)
 
 		if len(contentMapped) == 0 {
 			filtered = append(filtered, ev)
@@ -997,7 +1002,7 @@ func (c *commandeer) handleEvents(watcher *watcher.Batcher,
 		if istemp {
 			continue
 		}
-		if c.hugo.Deps.SourceSpec.IgnoreFile(ev.Name) {
+		if c.hugo().Deps.SourceSpec.IgnoreFile(ev.Name) {
 			continue
 		}
 		// Sometimes during rm -rf operations a '"": REMOVE' is triggered. Just ignore these
@@ -1073,7 +1078,7 @@ func (c *commandeer) handleEvents(watcher *watcher.Batcher,
 			// force refresh when more than one file
 			if len(staticEvents) == 1 {
 				ev := staticEvents[0]
-				path := c.hugo.BaseFs.SourceFilesystems.MakeStaticPathRelative(ev.Name)
+				path := c.hugo().BaseFs.SourceFilesystems.MakeStaticPathRelative(ev.Name)
 				path = c.firstPathSpec().RelURL(helpers.ToSlashTrimLeading(path), false)
 				livereload.RefreshPath(path)
 			} else {
@@ -1119,7 +1124,7 @@ func (c *commandeer) handleEvents(watcher *watcher.Batcher,
 
 				if navigate {
 					if onePageName != "" {
-						p = c.hugo.GetContentPage(onePageName)
+						p = c.hugo().GetContentPage(onePageName)
 					}
 				}
 
