@@ -1,6 +1,7 @@
 package hugolib
 
 import (
+	"image/jpeg"
 	"io"
 	"path/filepath"
 	"runtime"
@@ -8,7 +9,10 @@ import (
 	"testing"
 	"unicode/utf8"
 
+	"github.com/gohugoio/hugo/output"
+
 	"github.com/gohugoio/hugo/parser/metadecoders"
+	"github.com/google/go-cmp/cmp"
 
 	"github.com/gohugoio/hugo/parser"
 	"github.com/pkg/errors"
@@ -36,10 +40,16 @@ import (
 
 	"github.com/gohugoio/hugo/resources/resource"
 
+	qt "github.com/frankban/quicktest"
 	"github.com/gohugoio/hugo/common/loggers"
 	"github.com/gohugoio/hugo/hugofs"
-	"github.com/stretchr/testify/assert"
-	"github.com/stretchr/testify/require"
+)
+
+var (
+	deepEqualsPages         = qt.CmpEquals(cmp.Comparer(func(p1, p2 *pageState) bool { return p1 == p2 }))
+	deepEqualsOutputFormats = qt.CmpEquals(cmp.Comparer(func(o1, o2 output.Format) bool {
+		return o1.Name == o2.Name && o1.MediaType.Type() == o2.MediaType.Type()
+	}))
 )
 
 type sitesBuilder struct {
@@ -50,7 +60,7 @@ type sitesBuilder struct {
 	T       testing.TB
 	depsCfg deps.DepsCfg
 
-	*require.Assertions
+	*qt.C
 
 	logger *loggers.Logger
 
@@ -58,6 +68,7 @@ type sitesBuilder struct {
 
 	// Used to test partial rebuilds.
 	changedFiles []string
+	removedFiles []string
 
 	// Aka the Hugo server mode.
 	running bool
@@ -101,11 +112,11 @@ func newTestSitesBuilder(t testing.TB) *sitesBuilder {
 		Separator:         " ",
 	}
 
-	return &sitesBuilder{T: t, Assertions: require.New(t), Fs: fs, configFormat: "toml", dumper: litterOptions}
+	return &sitesBuilder{T: t, C: qt.New(t), Fs: fs, configFormat: "toml", dumper: litterOptions}
 }
 
 func newTestSitesBuilderFromDepsCfg(t testing.TB, d deps.DepsCfg) *sitesBuilder {
-	assert := require.New(t)
+	c := qt.New(t)
 
 	litterOptions := litter.Options{
 		HidePrivateFields: true,
@@ -113,7 +124,7 @@ func newTestSitesBuilderFromDepsCfg(t testing.TB, d deps.DepsCfg) *sitesBuilder
 		Separator:         " ",
 	}
 
-	b := &sitesBuilder{T: t, Assertions: assert, depsCfg: d, Fs: d.Fs, dumper: litterOptions}
+	b := &sitesBuilder{T: t, C: c, depsCfg: d, Fs: d.Fs, dumper: litterOptions}
 	workingDir := d.Cfg.GetString("workingDir")
 
 	b.WithWorkingDir(workingDir)
@@ -177,7 +188,7 @@ func (s *sitesBuilder) WithViper(v *viper.Viper) *sitesBuilder {
 	// Write to a config file to make sure the tests follow the same code path.
 	var buff bytes.Buffer
 	m := v.AllSettings()
-	s.Assertions.NoError(parser.InterfaceToConfig(m, metadecoders.TOML, &buff))
+	s.Assert(parser.InterfaceToConfig(m, metadecoders.TOML, &buff), qt.IsNil)
 	return s.WithConfigFile("toml", buff.String())
 }
 
@@ -323,13 +334,13 @@ lag = "lag"
 func (s *sitesBuilder) WithSunset(in string) {
 	// Write a real image into one of the bundle above.
 	src, err := os.Open(filepath.FromSlash("testdata/sunset.jpg"))
-	s.NoError(err)
+	s.Assert(err, qt.IsNil)
 
-	out, err := s.Fs.Source.Create(filepath.FromSlash(in))
-	s.NoError(err)
+	out, err := s.Fs.Source.Create(filepath.FromSlash(filepath.Join(s.workingDir, in)))
+	s.Assert(err, qt.IsNil)
 
 	_, err = io.Copy(out, src)
-	s.NoError(err)
+	s.Assert(err, qt.IsNil)
 
 	out.Close()
 	src.Close()
@@ -376,15 +387,22 @@ func (s *sitesBuilder) WithI18nAdded(filenameContent ...string) *sitesBuilder {
 }
 
 func (s *sitesBuilder) EditFiles(filenameContent ...string) *sitesBuilder {
-	var changedFiles []string
 	for i := 0; i < len(filenameContent); i += 2 {
 		filename, content := filepath.FromSlash(filenameContent[i]), filenameContent[i+1]
-		changedFiles = append(changedFiles, filename)
-		writeSource(s.T, s.Fs, s.absFilename(filename), content)
+		absFilename := s.absFilename(filename)
+		s.changedFiles = append(s.changedFiles, absFilename)
+		writeSource(s.T, s.Fs, absFilename, content)
 
 	}
-	s.changedFiles = changedFiles
+	return s
+}
 
+func (s *sitesBuilder) RemoveFiles(filenames ...string) *sitesBuilder {
+	for _, filename := range filenames {
+		absFilename := s.absFilename(filename)
+		s.removedFiles = append(s.removedFiles, absFilename)
+		s.Assert(s.Fs.Source.Remove(absFilename), qt.IsNil)
+	}
 	return s
 }
 
@@ -476,6 +494,8 @@ func (s *sitesBuilder) CreateSitesE() error {
 		return errors.Wrap(err, "failed to load config")
 	}
 
+	s.Fs.Destination = hugofs.NewCreateCountingFs(s.Fs.Destination)
+
 	depsCfg := s.depsCfg
 	depsCfg.Fs = s.Fs
 	depsCfg.Cfg = s.Cfg
@@ -510,23 +530,27 @@ func (s *sitesBuilder) BuildFail(cfg BuildCfg) *sitesBuilder {
 }
 
 func (s *sitesBuilder) changeEvents() []fsnotify.Event {
-	if len(s.changedFiles) == 0 {
-		return nil
-	}
 
-	events := make([]fsnotify.Event, len(s.changedFiles))
-	// TODO(bep) remove?
-	for i, v := range s.changedFiles {
-		events[i] = fsnotify.Event{
+	var events []fsnotify.Event
+
+	for _, v := range s.changedFiles {
+		events = append(events, fsnotify.Event{
 			Name: v,
 			Op:   fsnotify.Write,
-		}
+		})
+	}
+	for _, v := range s.removedFiles {
+		events = append(events, fsnotify.Event{
+			Name: v,
+			Op:   fsnotify.Remove,
+		})
 	}
 
 	return events
 }
 
 func (s *sitesBuilder) build(cfg BuildCfg, shouldFail bool) *sitesBuilder {
+	s.Helper()
 	defer func() {
 		s.changedFiles = nil
 	}()
@@ -576,10 +600,10 @@ date: "2018-02-28"
 			"content/sect/doc1.nn.md", contentTemplate,
 		}
 
-		listTemplateCommon = "{{ $p := .Paginator }}{{ $p.PageNumber }}|{{ .Title }}|{{ i18n \"hello\" }}|{{ .Permalink }}|Pager: {{ template \"_internal/pagination.html\" . }}|Kind: {{ .Kind }}|Content: {{ .Content }}"
+		listTemplateCommon = "{{ $p := .Paginator }}{{ $p.PageNumber }}|{{ .Title }}|{{ i18n \"hello\" }}|{{ .Permalink }}|Pager: {{ template \"_internal/pagination.html\" . }}|Kind: {{ .Kind }}|Content: {{ .Content }}|Len Pages: {{ len .Pages }}|Len RegularPages: {{ len .RegularPages }}| HasParent: {{ if .Parent }}YES{{ else }}NO{{ end }}"
 
 		defaultTemplates = []string{
-			"_default/single.html", "Single: {{ .Title }}|{{ i18n \"hello\" }}|{{.Language.Lang}}|RelPermalink: {{ .RelPermalink }}|Permalink: {{ .Permalink }}|{{ .Content }}|Resources: {{ range .Resources }}{{ .MediaType }}: {{ .RelPermalink}} -- {{ end }}|Summary: {{ .Summary }}|Truncated: {{ .Truncated }}",
+			"_default/single.html", "Single: {{ .Title }}|{{ i18n \"hello\" }}|{{.Language.Lang}}|RelPermalink: {{ .RelPermalink }}|Permalink: {{ .Permalink }}|{{ .Content }}|Resources: {{ range .Resources }}{{ .MediaType }}: {{ .RelPermalink}} -- {{ end }}|Summary: {{ .Summary }}|Truncated: {{ .Truncated }}|Parent: {{ .Parent.Title }}",
 			"_default/list.html", "List Page " + listTemplateCommon,
 			"index.html", "{{ $p := .Paginator }}Default Home Page {{ $p.PageNumber }}: {{ .Title }}|{{ .IsHome }}|{{ i18n \"hello\" }}|{{ .Permalink }}|{{  .Site.Data.hugo.slogan }}|String Resource: {{ ( \"Hugo Pipes\" | resources.FromString \"text/pipes.txt\").RelPermalink  }}",
 			"index.fr.html", "{{ $p := .Paginator }}French Home Page {{ $p.PageNumber }}: {{ .Title }}|{{ .IsHome }}|{{ i18n \"hello\" }}|{{ .Permalink }}|{{  .Site.Data.hugo.slogan }}|String Resource: {{ ( \"Hugo Pipes\" | resources.FromString \"text/pipes.txt\").RelPermalink  }}",
@@ -630,10 +654,6 @@ func (s *sitesBuilder) Fatalf(format string, args ...interface{}) {
 	s.T.Fatalf(format, args...)
 }
 
-func stackTrace() string {
-	return strings.Join(assert.CallerInfo(), "\n\r\t\t\t")
-}
-
 func (s *sitesBuilder) AssertFileContentFn(filename string, f func(s string) bool) {
 	s.T.Helper()
 	content := s.FileContent(filename)
@@ -649,13 +669,36 @@ func (s *sitesBuilder) AssertHome(matches ...string) {
 func (s *sitesBuilder) AssertFileContent(filename string, matches ...string) {
 	s.T.Helper()
 	content := s.FileContent(filename)
-	for _, match := range matches {
-		if !strings.Contains(content, match) {
-			s.Fatalf("No match for %q in content for %s\n%s\n%q", match, filename, content, content)
+	for _, m := range matches {
+		lines := strings.Split(m, "\n")
+		for _, match := range lines {
+			match = strings.TrimSpace(match)
+			if match == "" {
+				continue
+			}
+			if !strings.Contains(content, match) {
+				s.Fatalf("No match for %q in content for %s\n%s\n%q", match, filename, content, content)
+			}
 		}
 	}
 }
 
+func (s *sitesBuilder) AssertImage(width, height int, filename string) {
+	filename = filepath.Join(s.workingDir, filename)
+	f, err := s.Fs.Destination.Open(filename)
+	s.Assert(err, qt.IsNil)
+	defer f.Close()
+	cfg, err := jpeg.DecodeConfig(f)
+	s.Assert(cfg.Width, qt.Equals, width)
+	s.Assert(cfg.Height, qt.Equals, height)
+}
+
+func (s *sitesBuilder) AssertNoDuplicateWrites() {
+	s.Helper()
+	d := s.Fs.Destination.(hugofs.DuplicatesReporter)
+	s.Assert(d.ReportDuplicates(), qt.Equals, "")
+}
+
 func (s *sitesBuilder) FileContent(filename string) string {
 	s.T.Helper()
 	filename = filepath.FromSlash(filename)
@@ -691,35 +734,48 @@ func (s *sitesBuilder) CheckExists(filename string) bool {
 	return destinationExists(s.Fs, filepath.Clean(filename))
 }
 
+func newTestHelper(cfg config.Provider, fs *hugofs.Fs, t testing.TB) testHelper {
+	return testHelper{
+		Cfg: cfg,
+		Fs:  fs,
+		C:   qt.New(t),
+	}
+}
+
 type testHelper struct {
 	Cfg config.Provider
 	Fs  *hugofs.Fs
-	T   testing.TB
+	*qt.C
 }
 
 func (th testHelper) assertFileContent(filename string, matches ...string) {
+	th.Helper()
 	filename = th.replaceDefaultContentLanguageValue(filename)
-	content := readDestination(th.T, th.Fs, filename)
+	content := readDestination(th, th.Fs, filename)
 	for _, match := range matches {
 		match = th.replaceDefaultContentLanguageValue(match)
-		require.True(th.T, strings.Contains(content, match), fmt.Sprintf("File no match for\n%q in\n%q:\n%s", strings.Replace(match, "%", "%%", -1), filename, strings.Replace(content, "%", "%%", -1)))
+		th.Assert(strings.Contains(content, match), qt.Equals, true, qt.Commentf(match+" not in: \n"+content))
 	}
 }
 
 func (th testHelper) assertFileContentRegexp(filename string, matches ...string) {
 	filename = th.replaceDefaultContentLanguageValue(filename)
-	content := readDestination(th.T, th.Fs, filename)
+	content := readDestination(th, th.Fs, filename)
 	for _, match := range matches {
 		match = th.replaceDefaultContentLanguageValue(match)
 		r := regexp.MustCompile(match)
-		require.True(th.T, r.MatchString(content), fmt.Sprintf("File no match for\n%q in\n%q:\n%s", strings.Replace(match, "%", "%%", -1), filename, strings.Replace(content, "%", "%%", -1)))
+		matches := r.MatchString(content)
+		if !matches {
+			fmt.Println(match+":\n", content)
+		}
+		th.Assert(matches, qt.Equals, true)
 	}
 }
 
 func (th testHelper) assertFileNotExist(filename string) {
 	exists, err := helpers.Exists(filename, th.Fs.Destination)
-	require.NoError(th.T, err)
-	require.False(th.T, exists)
+	th.Assert(err, qt.IsNil)
+	th.Assert(exists, qt.Equals, false)
 }
 
 func (th testHelper) replaceDefaultContentLanguageValue(value string) string {
@@ -778,14 +834,16 @@ func newTestSitesFromConfig(t testing.TB, afs afero.Fs, tomlConfig string, layou
 		t.Fatalf("Layouts must be provided in pairs")
 	}
 
+	c := qt.New(t)
+
 	writeToFs(t, afs, filepath.Join("content", ".gitkeep"), "")
 	writeToFs(t, afs, "config.toml", tomlConfig)
 
 	cfg, err := LoadConfigDefault(afs)
-	require.NoError(t, err)
+	c.Assert(err, qt.IsNil)
 
 	fs := hugofs.NewFrom(afs, cfg)
-	th := testHelper{cfg, fs, t}
+	th := newTestHelper(cfg, fs, t)
 
 	for i := 0; i < len(layoutPathContentPairs); i += 2 {
 		writeSource(t, fs, layoutPathContentPairs[i], layoutPathContentPairs[i+1])
@@ -793,14 +851,14 @@ func newTestSitesFromConfig(t testing.TB, afs afero.Fs, tomlConfig string, layou
 
 	h, err := NewHugoSites(deps.DepsCfg{Fs: fs, Cfg: cfg})
 
-	require.NoError(t, err)
+	c.Assert(err, qt.IsNil)
 
 	return th, h
 }
 
-func createWithTemplateFromNameValues(additionalTemplates ...string) func(templ tpl.TemplateHandler) error {
+func createWithTemplateFromNameValues(additionalTemplates ...string) func(templ tpl.TemplateManager) error {
 
-	return func(templ tpl.TemplateHandler) error {
+	return func(templ tpl.TemplateManager) error {
 		for i := 0; i < len(additionalTemplates); i += 2 {
 			err := templ.AddTemplate(additionalTemplates[i], additionalTemplates[i+1])
 			if err != nil {
@@ -813,6 +871,7 @@ func createWithTemplateFromNameValues(additionalTemplates ...string) func(templ
 
 // TODO(bep) replace these with the builder
 func buildSingleSite(t testing.TB, depsCfg deps.DepsCfg, buildCfg BuildCfg) *Site {
+	t.Helper()
 	return buildSingleSiteExpected(t, false, false, depsCfg, buildCfg)
 }
 
@@ -823,23 +882,23 @@ func buildSingleSiteExpected(t testing.TB, expectSiteInitEror, expectBuildError
 	err := b.CreateSitesE()
 
 	if expectSiteInitEror {
-		require.Error(t, err)
+		b.Assert(err, qt.Not(qt.IsNil))
 		return nil
 	} else {
-		require.NoError(t, err)
+		b.Assert(err, qt.IsNil)
 	}
 
 	h := b.H
 
-	require.Len(t, h.Sites, 1)
+	b.Assert(len(h.Sites), qt.Equals, 1)
 
 	if expectBuildError {
-		require.Error(t, h.Build(buildCfg))
+		b.Assert(h.Build(buildCfg), qt.Not(qt.IsNil))
 		return nil
 
 	}
 
-	require.NoError(t, h.Build(buildCfg))
+	b.Assert(h.Build(buildCfg), qt.IsNil)
 
 	return h.Sites[0]
 }
@@ -873,13 +932,12 @@ func content(c resource.ContentProvider) string {
 
 func dumpPages(pages ...page.Page) {
 	fmt.Println("---------")
-	for i, p := range pages {
+	for _, p := range pages {
 		var meta interface{}
 		if p.File() != nil && p.File().FileInfo() != nil {
 			meta = p.File().FileInfo().Meta()
 		}
-		fmt.Printf("%d: Kind: %s Title: %-10s RelPermalink: %-10s Path: %-10s sections: %s Lang: %s Meta: %v\n",
-			i+1,
+		fmt.Printf("Kind: %s Title: %-10s RelPermalink: %-10s Path: %-10s sections: %s Lang: %s Meta: %v\n",
 			p.Kind(), p.Title(), p.RelPermalink(), p.Path(), p.SectionsPath(), p.Lang(), meta)
 	}
 }
@@ -916,10 +974,6 @@ func isCI() bool {
 	return os.Getenv("CI") != ""
 }
 
-func isGo111() bool {
-	return strings.Contains(runtime.Version(), "1.11")
-}
-
 // See https://github.com/golang/go/issues/19280
 // Not in use.
 var parallelEnabled = true
