@@ -24,7 +24,8 @@ import (
 	"reflect"
 	"strings"
 	"sync"
-	texttemplate "text/template"
+
+	texttemplate "github.com/gohugoio/hugo/tpl/internal/go_templates/texttemplate"
 
 	"github.com/gohugoio/hugo/helpers"
 
@@ -115,9 +116,9 @@ func (ns *Namespace) Include(name string, contextList ...interface{}) (interface
 		return "", fmt.Errorf("partial %q not found", name)
 	}
 
-	var info tpl.Info
-	if ip, ok := templ.(tpl.TemplateInfoProvider); ok {
-		info = ip.TemplateInfo()
+	var info tpl.ParseInfo
+	if ip, ok := templ.(tpl.Info); ok {
+		info = ip.ParseInfo()
 	}
 
 	var w io.Writer
@@ -138,7 +139,7 @@ func (ns *Namespace) Include(name string, contextList ...interface{}) (interface
 		w = b
 	}
 
-	if err := templ.Execute(w, context); err != nil {
+	if err := ns.deps.Tmpl.Execute(templ, w, context); err != nil {
 		return "", err
 	}
 
